@@ -40,10 +40,10 @@ to another broker. Either this should be added, or some clear exception
 should be raised to help the caller make this happen.
 """
 
-from session import BaseListener, Session, SessionMonitor, SessionException
-from util import DatabaseAction, startProcess, killProcess, doDatabaseAction, queryEngine
+from .session import BaseListener, Session, SessionMonitor, SessionException
+from .util import DatabaseAction, startProcess, killProcess, doDatabaseAction, queryEngine, logString
 
-import time, json
+import time, json, socket
 from threading import Event, Lock
 import xml.etree.ElementTree as ElementTree
 
@@ -57,10 +57,10 @@ class Domain(BaseListener):
 
     def __init__(self, broker_addr, domain_user, domain_pwd, listener=None):
         """
-        @type broker_addr str
-        @type domain_user str
-        @type domain_pwd str
-        @type listener
+        :type broker_addr str
+        :type domain_user str
+        :type domain_pwd str
+        :type listener
         """
         if not domain_pwd:
             raise Exception("A password is required to join a domain")
@@ -72,17 +72,19 @@ class Domain(BaseListener):
         self.__password = domain_pwd
         self.__listener = listener
         self.__peers = dict()
-        """ @type : dict[str, Peer] """
+        """ :type : dict[str, Peer] """
+        self.__peers_by_addr = dict()
+        """ :type : dict[str, Peer] """
         self.__databases = dict()
-        """ @type : dict[str, Database] """
+        """ :type : dict[str, Database] """
 
         self.__monitor = SessionMonitor(self.__session, self)
         
         # These will be set in handle status after joining the domain 
         self.__domain_name = None
-        """ @type : str """
+        """ :type : str """
         self.__entry_peer = None
-        """ @type : Peer """
+        """ :type : Peer """
 
         try:
             self.__session.doConnect()
@@ -120,18 +122,49 @@ class Domain(BaseListener):
         """Return the domain name."""
         return self.__domain_name
 
-    def find_peer(self, address):
+    def find_peer(self, address, port=None):
         """
         Find a peer by address
-        @type: address str
-        @rtype: Peer
+        :type: address str
+        :type: port int or str
+        :rtype: Peer
         """
+        if port is None:
+            if ":" in address:
+                address, port = address.split(':', 2)
+            else:
+                port = self.__entry_peer.port
+        else:
+            if ":" in address:
+                address, _ = address.split(':', 2)
+
+        ip = socket.gethostbyname(address)
+        inet_sock_addr = ":".join([ip, str(port)])
+        try:
+            return self.__peers_by_addr[inet_sock_addr]
+        except Exception as exception:
+            print(exception.message)
+
+        session = Session(address, port=port, service="Identity")
+        session.authorize(self.__user, self.__password)
+        response = session.doRequest()
+        try:
+            root = ElementTree.fromstring(response)
+            if self.__domain_name != root.get("Domain"):
+                return None
+            peer =  self.get_peer(root.get("AgentId"))
+            if peer:
+                self.__peers_by_addr[peer._get_normalized_addr()] = peer
+            return peer
+        except Exception as exception:
+            print(exception.message)
+            return None
 
     def get_peer(self, agent_id):
         """
         Return a peer for a given agent_id.
-        @type agent_id str
-        @rtype: Peer
+        :type agent_id str
+        :rtype: Peer
         """
         return self.__peers.get(agent_id)
 
@@ -139,7 +172,7 @@ class Domain(BaseListener):
     def peers(self):
         """
         Return a list of all peers in the domain.
-        @rtype: list[Peer]
+        :rtype: list[Peer]
         """
         return self.__peers.values()
 
@@ -147,15 +180,15 @@ class Domain(BaseListener):
     def entry_peer(self):
         """
         Return the peer that was used to enter the domain.
-        @rtype: Peer
+        :rtype: Peer
         """
         return self.__entry_peer
 
     def get_database(self, name):
         """
         Return a database by name
-        @type name str
-        @rtype: Database
+        :type name str
+        :rtype: Database
         """
         return self.__databases.get(name)
 
@@ -163,7 +196,7 @@ class Domain(BaseListener):
     def databases(self):
         """
         Return a list of databases in the domain
-        @rtype: list[Database]
+        :rtype: list[Database]
         """
         return self.__databases.values()
 
@@ -193,12 +226,12 @@ class Domain(BaseListener):
         response = self._send_domain_message(**Template.build_list_request())
         return Template.from_list_message(response)
 
-    def create_description(self, name, template_name, variables, dba_user, dba_password):
-        response = self._send_domain_message(**Description.build_create_request(name, template_name, variables, dba_user, dba_password))
+    def create_description(self, name, template_name, variables, dba_user, dba_password, db_opts=None):
+        response = self._send_domain_message(**Description.build_create_request(name, template_name, variables, dba_user, dba_password, db_opts))
         return ElementTree.fromstring(response).tag == Description.success_message
 
-    def update_description(self, name, template_name, variables):
-        response = self._send_domain_message(**Description.build_update_request(name, template_name, variables))
+    def update_description(self, name, template_name, variables, db_opts=None):
+        response = self._send_domain_message(**Description.build_update_request(name, template_name, variables, db_opts))
         return ElementTree.fromstring(response).tag == Description.success_message
 
     def delete_description(self, name):
@@ -228,7 +261,7 @@ class Domain(BaseListener):
         graceful -- (default True) means that the database will first
         be quiesced and then shutdown.
         """
-        for database in self.__databases.itervalues():
+        for database in list(self.__databases.values()):
             database.shutdown(graceful)
 
     def message_received(self, root):
@@ -315,6 +348,8 @@ class Domain(BaseListener):
                 name = child.get("Name")
                 if self.__listener:
                     try:
+                        if name not in self.__databases:
+                            self.__databases[name] = Database(self, name)
                         self.__listener.database_joined(self.__databases[name])
                     except AttributeError:
                         pass
@@ -328,6 +363,7 @@ class Domain(BaseListener):
     def __peer_joined(self, peer):
         """Called when a peer joins the domain."""
         self.__peers[peer.id] = peer
+        self.__peers_by_addr[peer._get_normalized_addr()] = peer
         if self.__listener:
             try:
                 self.__listener.peer_joined(peer)
@@ -337,6 +373,7 @@ class Domain(BaseListener):
     def __peer_left(self, peer):
         """Called when a peer leaves the domain."""
         del self.__peers[peer.id]
+        del self.__peers_by_addr[peer._get_normalized_addr()]
         if self.__listener:
             try:
                 self.__listener.peer_left(peer)
@@ -407,18 +444,18 @@ class Domain(BaseListener):
         self.__session.send(ElementTree.tostring(root))
 
 
-class Peer:
+class Peer(object):
     """Represents a peer (or host) in the domain."""
 
     def __init__(self, domain, address, agent_id, broker=False, port=48004, hostname=None, version=None):
         """
-        @type domain Domain
-        @type address str
-        @type agent_id str
-        @type broker bool
-        @type port int
-        @type hostname str
-        @type version str
+        :type domain Domain
+        :type address str
+        :type agent_id str
+        :type broker bool
+        :type port int
+        :type hostname str
+        :type version str
         """
         self.__domain = domain
         self.__address = address
@@ -430,6 +467,7 @@ class Peer:
         self.__processes = dict()
         self.__version = version
         self.__start_id_slots = dict()
+        self.__inet_sock_addr = None
 
     @staticmethod
     def from_message(domain, peer_element):
@@ -457,7 +495,7 @@ class Peer:
     def domain(self):
         """
         Return the domain that contains this peer.
-        @rtype: Domain
+        :rtype: Domain
         """
         return self.__domain
 
@@ -465,7 +503,7 @@ class Peer:
     def address(self):
         """
         Return the address of this peer.
-        @rtype: str
+        :rtype: str
         """
         return self.__address
 
@@ -473,7 +511,7 @@ class Peer:
     def connect_str(self):
         """
         Return the connect string for this peer.
-        @rtype: str
+        :rtype: str
         """
         return self.__address + ":" + str(self.__port)
 
@@ -481,7 +519,7 @@ class Peer:
     def port(self):
         """
         Return the port that this peer is using.
-        @rtype: int
+        :rtype: int
         """
         return self.__port
 
@@ -489,7 +527,7 @@ class Peer:
     def id(self):
         """
         Return the id of this peer (agent_id).
-        @rtype: str
+        :rtype: str
         """
         return self.__id
 
@@ -497,7 +535,7 @@ class Peer:
     def hostname(self):
         """
         Return the hostname of this peer.
-        @rtype: str
+        :rtype: str
         """
         return self.__hostname
     
@@ -505,7 +543,7 @@ class Peer:
     def version(self):
         """
         Return the NuoDB release version of this peer.
-        @rtype: str
+        :rtype: str
         """
         return self.__version
 
@@ -513,7 +551,7 @@ class Peer:
     def is_broker(self):
         """
         Return True if this peer is a broker.
-        @rtype: bool
+        :rtype: bool
         """
         return self.__is_broker
 
@@ -521,7 +559,7 @@ class Peer:
     def tags(self):
         """
         Return all host tags
-        @rtype: dict[str,str]
+        :rtype: dict[str,str]
         """
         message = self.__domain._send_domain_message("Tag", {'Action': 'GetHostTags', 'AgentId': self.id})
         tags = ElementTree.fromstring(message)
@@ -534,15 +572,15 @@ class Peer:
     def get_tag(self, tag):
         """
         Return host tag
-        @rtype: str
+        :rtype: str
         """
         return self.tags[tag]
 
     def set_tag(self, key, value):
         """
         Set host tag
-        @type key str
-        @type value str
+        :type key str
+        :type value str
         """
         element = ElementTree.fromstring("<Tag Key=\"%s\" Value=\"%s\"/>" % (key, value))
         self.__domain._send_domain_message("Tag", {'Action': 'SetHostTags', 'AgentId': self.id}, children=[element])
@@ -550,11 +588,14 @@ class Peer:
     def delete_tag(self, key):
         """
         Delete host tag
-        @type key str
+        :type key str
         """
         element = ElementTree.fromstring("<Tag Key=\"%s\"/>" % (key))
         self.__domain._send_domain_message("Tag", {'Action': 'DeleteHostTags', 'AgentId': self.id}, children=[element])
 
+    def log_string(self, message):
+        return logString(self.connect_str, self.__domain.user, self.__domain.password, message)
+
     def start_transaction_engine(self, db_name, options=None, wait_seconds=None):
         """Start a transaction engine on this peer for a given database.
         
@@ -571,10 +612,10 @@ class Peer:
         until a response is received indicating success or failure. If the
         time elapses without a response a SessionException will be raised.
 
-        @type db_name str
-        @type options list[tuple[str]]
-        @type wait_seconds int
-        @rtype: Process
+        :type db_name str
+        :type options list[tuple[str]]
+        :type wait_seconds int
+        :rtype: Process
         """
         return self.__start_process(db_name, options, wait_seconds)
 
@@ -596,15 +637,17 @@ class Peer:
         until a response is received indicating success or failure. If the
         time elapses without a response, a SessionException will be raised.
 
-        @type db_name str
-        @type archive str
-        @type initialize bool
-        @type options list[tuple[str]]
-        @type wait_seconds int
-        @rtype: Process
+        :type db_name str
+        :type archive str
+        :type initialize bool
+        :type options list[tuple[str]]
+        :type wait_seconds int
+        :rtype: Process
         """
         if not options:
             options = []
+        else:
+            options = list(options)
 
         options.append(("--archive", archive))
 
@@ -615,10 +658,10 @@ class Peer:
 
     def __start_process(self, db_name, options, wait_seconds):
         """
-        @type db_name str
-        @type options list[tuple[str]]
-        @type wait_seconds int
-        @rtype: Process | None
+        :type db_name str
+        :type options list[tuple[str]]
+        :type wait_seconds int
+        :rtype: Process | None
         """
         if wait_seconds is None:
             startProcess(self.connect_str, self.__domain.user, self.__domain.password, db_name, options)
@@ -678,7 +721,7 @@ class Peer:
         this method will only return the subset of processes that are on this 
         peer.
 
-        @rtype: list[Process]
+        :rtype: list[Process]
         """
         if db_name is None:
             return self.__processes.values()
@@ -692,41 +735,52 @@ class Peer:
 
     def _get_process(self, pid):
         """
-        @type pid int
-        @rtype: Process
+        :type pid int
+        :rtype: Process
         """
         return self.__processes.get(pid)
 
     def _add_process(self, process):
         """
-        @type process Process
+        :type process Process
         """
         self.__processes[process.pid] = process
 
     def _remove_process(self, process):
         """
-        @type process Process
+        :type process Process
         """
         try:
             del self.__processes[process.pid]
-        except:
+        except Exception as exception:
+            print(exception.message)
             pass
 
+    def _get_normalized_addr(self):
+        """
+        Return ip_address:port
+        :rtype: str
+        """
+        if self.__inet_sock_addr is None:
+            ip = socket.gethostbyname(self.__address)
+            inet_sock_addr = ":".join([ip, str(self.__port)])
+            self.__inet_sock_addr = inet_sock_addr
+        return self.__inet_sock_addr
 
-class Database:
+class Database(object):
     
     """Represents a NuoDB database."""
 
     def __init__(self, domain, name):
         """
-        @type domain Domain
-        @type name str
+        :type domain Domain
+        :type name str
         """
         self.__domain = domain
         self.__name = name
 
         self.__processes = dict()
-        """ @type : dict[str, Process] """
+        """ :type : dict[str, Process] """
 
     def __hash__(self):
         return hash(self.__name)
@@ -746,7 +800,7 @@ class Database:
     def domain(self):
         """
         Return the domain that contains this database.
-        @rtype: Domain
+        :rtype: Domain
         """
         return self.__domain
 
@@ -754,7 +808,7 @@ class Database:
     def name(self):
         """
         Return the name of this database.
-        @rtype: str
+        :rtype: str
         """
         return self.__name
 
@@ -814,7 +868,7 @@ class Database:
         failure_count = 0
         failure_text = ""
 
-        for process in self.__processes.values():
+        for process in list(self.__processes.values()):
             if process.is_transactional:
                 try:
                     if graceful:
@@ -822,18 +876,18 @@ class Database:
                     else:
                         process.kill()
                     
-                except Exception, e:
+                except Exception as e:
                     failure_count = failure_count + 1
                     failure_text = failure_text + str(e) + "\n"
 
-        for process in self.__processes.values():
+        for process in list(self.__processes.values()):
             if not process.is_transactional:
                 try:
                     if graceful:
                         process.shutdown()
                     else:
                         process.kill()
-                except Exception, e:
+                except Exception as e:
                     failure_count = failure_count + 1
                     failure_text = failure_text + str(e) + "\n"
 
@@ -903,21 +957,21 @@ class Database:
 
         return False
 
-class Process:
+class Process(object):
     
     """Represents a NuoDB process (TE or SM)"""
 
     def __init__(self, peer, database, port, pid, transactional, status, hostname, version, node_id):
         """
-        @type peer Peer
-        @type database Database
-        @type port int
-        @type pid int
-        @type transactional bool
-        @type status str
-        @type hostname str
-        @type version str
-        @type node_id int
+        :type peer Peer
+        :type database Database
+        :type port int
+        :type pid int
+        :type transactional bool
+        :type status str
+        :type hostname str
+        :type version str
+        :type node_id int
         """
         self.__peer = peer
         self.__database = database
@@ -1080,7 +1134,7 @@ class Process:
     # to send this over the existing connection, but for RC1 that's one too
     # many moving pieces to implement and test
     def query(self, query_type, msg_body=None):
-        session = Session(self.peer.connect_str, service="Manager")
+        session = Session(self.peer.domain.entry_peer.connect_str, service="Manager")
         session.authorize(self.peer.domain.user, self.peer.domain.password)
         pwd_response = session.doRequest(attributes={"Type": "GetDatabaseCredentials",
                                                      "Database": self.database.name})
@@ -1091,7 +1145,7 @@ class Process:
         return queryEngine(self.address, self.port, query_type, pwd, msg_body)
 
 
-class Template:
+class Template(object):
     success_message = "Success"
 
     @staticmethod
@@ -1154,9 +1208,9 @@ class Template:
 
     def __init__(self, name, summary, requirements):
         """
-        @type name str
-        @type summary str
-        @type requirements str
+        :type name str
+        :type summary str
+        :type requirements str
         """
         self._name = name
         self._summary = summary
@@ -1175,11 +1229,11 @@ class Template:
         return self._requirements
 
 
-class Description:
+class Description(object):
     success_message = "Success"
 
     @staticmethod
-    def build_create_request(name, template_name, variables, dba_user, dba_password):
+    def build_create_request(name, template_name, variables, dba_user, dba_password, db_options=None):
         template_element = ElementTree.Element("Template")
         template_element.text = template_name
         variables_element = ElementTree.Element("Variables")
@@ -1188,15 +1242,22 @@ class Description:
             variable_child.set("Key", key)
             variable_child.text = variables[key]
 
+        if db_options is None: db_options = {}
+        options_element = ElementTree.Element("Options")
+        for key in db_options:
+            option_child = ElementTree.SubElement(options_element, "Option")
+            option_child.set("Key", key)
+            option_child.text = db_options[key]
+
         return {"service": "Description",
                 "attributes": {'Action': 'CreateDescription',
                                'DatabaseName': name,
                                'DbaUser': dba_user,
                                'DbaPassword': dba_password},
-                "children": [template_element, variables_element]}
+                "children": [template_element, variables_element, options_element]}
 
     @staticmethod
-    def build_update_request(name, template_name, variables):
+    def build_update_request(name, template_name, variables, db_options=None):
         template_element = ElementTree.Element("Template")
         template_element.text = template_name
         variables_element = ElementTree.Element("Variables")
@@ -1205,10 +1266,17 @@ class Description:
             variable_child.set("Key", key)
             variable_child.text = variables[key]
 
+        if db_options is None: db_options = {}
+        options_element = ElementTree.Element("Options")
+        for key in db_options:
+            option_child = ElementTree.SubElement(options_element, "Option")
+            option_child.set("Key", key)
+            option_child.text = db_options[key]
+
         return {"service": "Description",
                 "attributes": {'Action': 'UpdateDescription',
                                'DatabaseName': name},
-                "children": [template_element, variables_element]}
+                "children": [template_element, variables_element, options_element]}
 
     @staticmethod
     def build_delete_request(name):
@@ -1253,7 +1321,12 @@ class Description:
         if status_element is not None:
             status = status_element.text
 
-        return Description(name, template_name, variables, status)
+        live_status = ""
+        live_status_element = root.find("LiveStatus")
+        if live_status_element is not None:
+            live_status = live_status_element.text
+
+        return Description(name, template_name, variables, status, live_status)
 
     @staticmethod
     def from_list_message(message):
@@ -1264,17 +1337,19 @@ class Description:
 
         return names
 
-    def __init__(self, name, template_name, variables, status):
+    def __init__(self, name, template_name, variables, status, live_status=""):
         """
-        @type name str
-        @type template_name str
-        @type variables dict[str,str]
-        @type status str
+        :type name str
+        :type template_name str
+        :type variables dict[str,str]
+        :type status str
+        :type live_status str
         """
         self._name = name
         self._template_name = template_name
         self._variables = variables
         self._status = status
+        self._live_status = live_status
 
     @property
     def name(self):
@@ -1290,4 +1365,8 @@ class Description:
 
     @property
     def status(self):
-        return self._status
\ No newline at end of file
+        return self._status
+
+    @property
+    def live_status(self):
+        return self._live_status
