@@ -4,10 +4,10 @@ __author__ = 'jgetto'
 class ResultSet(object):
     def __init__(self, handle, col_count, initial_results, complete):
         """
-        @type handle int
-        @type col_count int
-        @type initial_results list
-        @type complete bool
+        :type handle int
+        :type col_count int
+        :type initial_results list
+        :type complete bool
         """
         self.handle = handle
         self.col_count = col_count
@@ -24,7 +24,7 @@ class ResultSet(object):
 
     def fetchone(self, session):
         """
-        @type session EncodedSession
+        :type session EncodedSession
         """
         if self.results_idx == len(self.results):
             if not self.complete:
@@ -35,3 +35,9 @@ class ResultSet(object):
         res = self.results[self.results_idx]
         self.results_idx += 1
         return res
+
+    def close(self, session):
+        """
+        :type session EncodedSession
+        """
+        session.close_result_set(self)
